@@ -1,24 +1,18 @@
 # -*- coding: utf-8 -*-
 ###
-# (C) Copyright (2012-2017) Hewlett Packard Enterprise Development LP
+# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+#   http://www.apache.org/licenses/LICENSE-2.0
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 ###
 """
 This module implements a common client for HPE OneView REST API.
@@ -51,6 +45,7 @@ from hpOneView.resources.networking.switch_types import SwitchTypes
 from hpOneView.resources.activity.tasks import Tasks
 from hpOneView.resources.settings.restores import Restores
 from hpOneView.resources.settings.scopes import Scopes
+from hpOneView.resources.settings.licenses import Licenses
 from hpOneView.resources.servers.enclosures import Enclosures
 from hpOneView.resources.servers.logical_enclosures import LogicalEnclosures
 from hpOneView.resources.servers.enclosure_groups import EnclosureGroups
@@ -69,7 +64,6 @@ from hpOneView.resources.uncategorized.unmanaged_devices import UnmanagedDevices
 from hpOneView.resources.networking.logical_downlinks import LogicalDownlinks
 from hpOneView.resources.facilities.power_devices import PowerDevices
 from hpOneView.resources.facilities.racks import Racks
-from hpOneView.resources.uncategorized.roles import Roles
 from hpOneView.resources.facilities.datacenters import Datacenters
 from hpOneView.resources.fc_sans.managed_sans import ManagedSANs
 from hpOneView.resources.fc_sans.san_managers import SanManagers
@@ -103,18 +97,29 @@ from hpOneView.resources.activity.events import Events
 from hpOneView.resources.uncategorized.os_deployment_plans import OsDeploymentPlans
 from hpOneView.resources.uncategorized.os_deployment_servers import OsDeploymentServers
 from hpOneView.resources.security.certificate_rabbitmq import CertificateRabbitMQ
+from hpOneView.resources.security.login_details import LoginDetails
+from hpOneView.resources.security.roles import Roles
 from hpOneView.resources.security.users import Users
+from hpOneView.resources.settings.appliance_device_read_community import ApplianceDeviceReadCommunity
+from hpOneView.resources.settings.appliance_device_snmp_v1_trap_destinations import ApplianceDeviceSNMPv1TrapDestinations
+from hpOneView.resources.settings.appliance_device_snmp_v3_trap_destinations import ApplianceDeviceSNMPv3TrapDestinations
+from hpOneView.resources.settings.appliance_device_snmp_v3_users import ApplianceDeviceSNMPv3Users
 from hpOneView.resources.settings.appliance_node_information import ApplianceNodeInformation
 from hpOneView.resources.settings.appliance_time_and_locale_configuration import ApplianceTimeAndLocaleConfiguration
+from hpOneView.resources.settings.versions import Versions
+from hpOneView.resources.hypervisors.hypervisor_managers import HypervisorManagers
+from hpOneView.resources.security.certificates_server import CertificatesServer
+from hpOneView.resources.hypervisors.hypervisor_cluster_profiles import HypervisorClusterProfiles
 
 ONEVIEW_CLIENT_INVALID_PROXY = 'Invalid Proxy format'
 
 
 class OneViewClient(object):
-    DEFAULT_API_VERSION = 300
+    DEFAULT_API_VERSION = 800
 
     def __init__(self, config):
-        self.__connection = connection(config["ip"], config.get('api_version', self.DEFAULT_API_VERSION))
+        self.__connection = connection(config["ip"], config.get('api_version', self.DEFAULT_API_VERSION), config.get('ssl_certificate', False),
+                                       config.get('timeout'))
         self.__image_streamer_ip = config.get("image_streamer_ip")
         self.__set_proxy(config)
         self.__connection.login(config["credentials"])
@@ -186,10 +191,18 @@ class OneViewClient(object):
         self.__os_deployment_servers = None
         self.__certificate_rabbitmq = None
         self.__users = None
+        self.__appliance_device_read_community = None
+        self.__appliance_device_snmp_v1_trap_destinations = None
+        self.__appliance_device_snmp_v3_trap_destinations = None
+        self.__appliance_device_snmp_v3_users = None
         self.__appliance_time_and_locale_configuration = None
         self.__appliance_node_information = None
+        self.__versions = None
         self.__backups = None
-        # TODO: Implement: con.set_trusted_ssl_bundle(args.cert)
+        self.__login_details = None
+        self.__licenses = None
+        self.__hypervisor_managers = None
+        self.__certificates_server = None
 
     @classmethod
     def from_json_file(cls, file_name):
@@ -213,7 +226,8 @@ class OneViewClient(object):
         Construct OneViewClient using environment variables.
 
         Allowed variables: ONEVIEWSDK_IP (required), ONEVIEWSDK_USERNAME (required), ONEVIEWSDK_PASSWORD (required),
-        ONEVIEWSDK_AUTH_LOGIN_DOMAIN, ONEVIEWSDK_API_VERSION, ONEVIEWSDK_IMAGE_STREAMER_IP, and ONEVIEWSDK_PROXY.
+        ONEVIEWSDK_AUTH_LOGIN_DOMAIN, ONEVIEWSDK_API_VERSION, ONEVIEWSDK_IMAGE_STREAMER_IP, ONEVIEWSDK_SESSIONID, ONEVIEWSDK_SSL_CERTIFICATE,
+        ONEVIEWSDK_CONNECTION_TIMEOUT and ONEVIEWSDK_PROXY.
 
         Returns:
             OneViewClient:
@@ -221,16 +235,20 @@ class OneViewClient(object):
         ip = os.environ.get('ONEVIEWSDK_IP', '')
         image_streamer_ip = os.environ.get('ONEVIEWSDK_IMAGE_STREAMER_IP', '')
         api_version = int(os.environ.get('ONEVIEWSDK_API_VERSION', OneViewClient.DEFAULT_API_VERSION))
+        ssl_certificate = os.environ.get('ONEVIEWSDK_SSL_CERTIFICATE', '')
         username = os.environ.get('ONEVIEWSDK_USERNAME', '')
         auth_login_domain = os.environ.get('ONEVIEWSDK_AUTH_LOGIN_DOMAIN', '')
         password = os.environ.get('ONEVIEWSDK_PASSWORD', '')
         proxy = os.environ.get('ONEVIEWSDK_PROXY', '')
+        sessionID = os.environ.get('ONEVIEWSDK_SESSIONID', '')
+        timeout = os.environ.get('ONEVIEWSDK_CONNECTION_TIMEOUT')
 
         config = dict(ip=ip,
                       image_streamer_ip=image_streamer_ip,
                       api_version=api_version,
-                      credentials=dict(userName=username, authLoginDomain=auth_login_domain, password=password),
-                      proxy=proxy)
+                      ssl_certificate=ssl_certificate,
+                      credentials=dict(userName=username, authLoginDomain=auth_login_domain, password=password, sessionID=sessionID),
+                      proxy=proxy, timeout=timeout)
 
         return cls(config)
 
@@ -279,7 +297,8 @@ class OneViewClient(object):
         """
         image_streamer = ImageStreamerClient(self.__image_streamer_ip,
                                              self.__connection.get_session_id(),
-                                             self.__connection._apiVersion)
+                                             self.__connection._apiVersion,
+                                             self.__connection._sslBundle)
 
         return image_streamer
 
@@ -316,10 +335,7 @@ class OneViewClient(object):
         Returns:
             ConnectionTemplates:
         """
-        if not self.__connection_templates:
-            self.__connection_templates = ConnectionTemplates(
-                self.__connection)
-        return self.__connection_templates
+        return ConnectionTemplates(self.__connection)
 
     @property
     def fc_networks(self):
@@ -329,9 +345,7 @@ class OneViewClient(object):
         Returns:
             FcNetworks:
         """
-        if not self.__fc_networks:
-            self.__fc_networks = FcNetworks(self.__connection)
-        return self.__fc_networks
+        return FcNetworks(self.__connection)
 
     @property
     def fcoe_networks(self):
@@ -341,9 +355,7 @@ class OneViewClient(object):
         Returns:
             FcoeNetworks:
         """
-        if not self.__fcoe_networks:
-            self.__fcoe_networks = FcoeNetworks(self.__connection)
-        return self.__fcoe_networks
+        return FcoeNetworks(self.__connection)
 
     @property
     def ethernet_networks(self):
@@ -353,9 +365,7 @@ class OneViewClient(object):
         Returns:
             EthernetNetworks:
         """
-        if not self.__ethernet_networks:
-            self.__ethernet_networks = EthernetNetworks(self.__connection)
-        return self.__ethernet_networks
+        return EthernetNetworks(self.__connection)
 
     @property
     def fabrics(self):
@@ -413,9 +423,7 @@ class OneViewClient(object):
         Returns:
             NetworkSets:
         """
-        if not self.__network_sets:
-            self.__network_sets = NetworkSets(self.__connection)
-        return self.__network_sets
+        return NetworkSets(self.__connection)
 
     @property
     def server_hardware(self):
@@ -425,9 +433,7 @@ class OneViewClient(object):
         Returns:
             ServerHardware:
         """
-        if not self.__server_hardware:
-            self.__server_hardware = ServerHardware(self.__connection)
-        return self.__server_hardware
+        return ServerHardware(self.__connection)
 
     @property
     def server_hardware_types(self):
@@ -437,10 +443,7 @@ class OneViewClient(object):
         Returns:
             ServerHardwareTypes:
         """
-        if not self.__server_hardware_types:
-            self.__server_hardware_types = ServerHardwareTypes(
-                self.__connection)
-        return self.__server_hardware_types
+        return ServerHardwareTypes(self.__connection)
 
     @property
     def id_pools_vsn_ranges(self):
@@ -546,9 +549,7 @@ class OneViewClient(object):
         Returns:
             SwitchTypes:
         """
-        if not self.__switch_types:
-            self.__switch_types = SwitchTypes(self.__connection)
-        return self.__switch_types
+        return SwitchTypes(self.__connection)
 
     @property
     def logical_switch_groups(self):
@@ -558,9 +559,7 @@ class OneViewClient(object):
         Returns:
             LogicalSwitchGroups:
         """
-        if not self.__logical_switch_groups:
-            self.__logical_switch_groups = LogicalSwitchGroups(self.__connection)
-        return self.__logical_switch_groups
+        return LogicalSwitchGroups(self.__connection)
 
     @property
     def logical_switches(self):
@@ -594,9 +593,7 @@ class OneViewClient(object):
         Returns:
             EnclosureGroups:
         """
-        if not self.__enclosure_groups:
-            self.__enclosure_groups = EnclosureGroups(self.__connection)
-        return self.__enclosure_groups
+        return EnclosureGroups(self.__connection)
 
     @property
     def enclosures(self):
@@ -606,9 +603,7 @@ class OneViewClient(object):
         Returns:
             Enclosures:
         """
-        if not self.__enclosures:
-            self.__enclosures = Enclosures(self.__connection)
-        return self.__enclosures
+        return Enclosures(self.__connection)
 
     @property
     def logical_enclosures(self):
@@ -618,9 +613,7 @@ class OneViewClient(object):
         Returns:
             LogicalEnclosures:
         """
-        if not self.__logical_enclosures:
-            self.__logical_enclosures = LogicalEnclosures(self.__connection)
-        return self.__logical_enclosures
+        return LogicalEnclosures(self.__connection)
 
     @property
     def metric_streaming(self):
@@ -642,9 +635,7 @@ class OneViewClient(object):
         Returns:
             Interconnects:
         """
-        if not self.__interconnects:
-            self.__interconnects = Interconnects(self.__connection)
-        return self.__interconnects
+        return Interconnects(self.__connection)
 
     @property
     def interconnect_types(self):
@@ -654,9 +645,7 @@ class OneViewClient(object):
         Returns:
             InterconnectTypes:
         """
-        if not self.__interconnect_types:
-            self.__interconnect_types = InterconnectTypes(self.__connection)
-        return self.__interconnect_types
+        return InterconnectTypes(self.__connection)
 
     @property
     def interconnect_link_topologies(self):
@@ -678,9 +667,7 @@ class OneViewClient(object):
         Returns:
             SasInterconnectTypes:
         """
-        if not self.__sas_interconnect_types:
-            self.__sas_interconnect_types = SasInterconnectTypes(self.__connection)
-        return self.__sas_interconnect_types
+        return SasInterconnectTypes(self.__connection)
 
     @property
     def internal_link_sets(self):
@@ -690,9 +677,7 @@ class OneViewClient(object):
         Returns:
             InternalLinkSets:
         """
-        if not self.__internal_link_sets:
-            self.__internal_link_sets = InternalLinkSets(self.__connection)
-        return self.__internal_link_sets
+        return InternalLinkSets(self.__connection)
 
     @property
     def logical_interconnect_groups(self):
@@ -702,10 +687,7 @@ class OneViewClient(object):
         Returns:
             LogicalInterconnectGroups:
         """
-        if not self.__logical_interconnect_groups:
-            self.__logical_interconnect_groups = LogicalInterconnectGroups(
-                self.__connection)
-        return self.__logical_interconnect_groups
+        return LogicalInterconnectGroups(self.__connection)
 
     @property
     def logical_interconnects(self):
@@ -715,10 +697,7 @@ class OneViewClient(object):
         Returns:
             LogicalInterconnects:
         """
-        if not self.__logical_interconnects:
-            self.__logical_interconnects = LogicalInterconnects(
-                self.__connection)
-        return self.__logical_interconnects
+        return LogicalInterconnects(self.__connection)
 
     @property
     def sas_logical_interconnects(self):
@@ -728,9 +707,7 @@ class OneViewClient(object):
         Returns:
             SasLogicalInterconnects:
         """
-        if not self.__sas_logical_interconnects:
-            self.__sas_logical_interconnects = SasLogicalInterconnects(self.__connection)
-        return self.__sas_logical_interconnects
+        return SasLogicalInterconnects(self.__connection)
 
     @property
     def logical_downlinks(self):
@@ -813,9 +790,7 @@ class OneViewClient(object):
         Returns:
             ServerProfiles:
         """
-        if not self.__server_profiles:
-            self.__server_profiles = ServerProfiles(self.__connection)
-        return self.__server_profiles
+        return ServerProfiles(self.__connection)
 
     @property
     def server_profile_templates(self):
@@ -825,9 +800,7 @@ class OneViewClient(object):
         Returns:
             ServerProfileTemplate:
         """
-        if not self.__server_profile_templates:
-            self.__server_profile_templates = ServerProfileTemplate(self.__connection)
-        return self.__server_profile_templates
+        return ServerProfileTemplate(self.__connection)
 
     @property
     def storage_systems(self):
@@ -837,9 +810,7 @@ class OneViewClient(object):
         Returns:
             StorageSystems:
         """
-        if not self.__storage_systems:
-            self.__storage_systems = StorageSystems(self.__connection)
-        return self.__storage_systems
+        return StorageSystems(self.__connection)
 
     @property
     def storage_pools(self):
@@ -849,9 +820,7 @@ class OneViewClient(object):
         Returns:
             StoragePools:
         """
-        if not self.__storage_pools:
-            self.__storage_pools = StoragePools(self.__connection)
-        return self.__storage_pools
+        return StoragePools(self.__connection)
 
     @property
     def storage_volume_templates(self):
@@ -861,9 +830,7 @@ class OneViewClient(object):
         Returns:
             StorageVolumeTemplates:
         """
-        if not self.__storage_volume_templates:
-            self.__storage_volume_templates = StorageVolumeTemplates(self.__connection)
-        return self.__storage_volume_templates
+        return StorageVolumeTemplates(self.__connection)
 
     @property
     def storage_volume_attachments(self):
@@ -873,9 +840,7 @@ class OneViewClient(object):
         Returns:
             StorageVolumeAttachments:
         """
-        if not self.__storage_volume_attachments:
-            self.__storage_volume_attachments = StorageVolumeAttachments(self.__connection)
-        return self.__storage_volume_attachments
+        return StorageVolumeAttachments(self.__connection)
 
     @property
     def firmware_drivers(self):
@@ -909,9 +874,7 @@ class OneViewClient(object):
         Returns:
             UplinkSets:
         """
-        if not self.__uplink_sets:
-            self.__uplink_sets = UplinkSets(self.__connection)
-        return self.__uplink_sets
+        return UplinkSets(self.__connection)
 
     @property
     def volumes(self):
@@ -921,9 +884,7 @@ class OneViewClient(object):
         Returns:
             Volumes:
         """
-        if not self.__volumes:
-            self.__volumes = Volumes(self.__connection)
-        return self.__volumes
+        return Volumes(self.__connection)
 
     @property
     def sas_logical_jbod_attachments(self):
@@ -945,9 +906,7 @@ class OneViewClient(object):
         Returns:
             ManagedSANs:
         """
-        if not self.__managed_sans:
-            self.__managed_sans = ManagedSANs(self.__connection)
-        return self.__managed_sans
+        return ManagedSANs(self.__connection)
 
     @property
     def migratable_vc_domains(self):
@@ -969,9 +928,7 @@ class OneViewClient(object):
         Returns:
             SasInterconnects:
         """
-        if not self.__sas_interconnects:
-            self.__sas_interconnects = SasInterconnects(self.__connection)
-        return self.__sas_interconnects
+        return SasInterconnects(self.__connection)
 
     @property
     def sas_logical_interconnect_groups(self):
@@ -981,9 +938,7 @@ class OneViewClient(object):
         Returns:
             SasLogicalInterconnectGroups:
         """
-        if not self.__sas_logical_interconnect_groups:
-            self.__sas_logical_interconnect_groups = SasLogicalInterconnectGroups(self.__connection)
-        return self.__sas_logical_interconnect_groups
+        return SasLogicalInterconnectGroups(self.__connection)
 
     @property
     def drive_enclosures(self):
@@ -1065,9 +1020,7 @@ class OneViewClient(object):
         Returns:
             OsDeploymentPlans:
         """
-        if not self.__os_deployment_plans:
-            self.__os_deployment_plans = OsDeploymentPlans(self.__connection)
-        return self.__os_deployment_plans
+        return OsDeploymentPlans(self.__connection)
 
     @property
     def os_deployment_servers(self):
@@ -1105,6 +1058,54 @@ class OneViewClient(object):
             self.__users = Users(self.__connection)
         return self.__users
 
+    @property
+    def appliance_device_read_community(self):
+        """
+        Gets the ApplianceDeviceReadCommunity API client.
+
+        Returns:
+            ApplianceDeviceReadCommunity:
+        """
+        if not self.__appliance_device_read_community:
+            self.__appliance_device_read_community = ApplianceDeviceReadCommunity(self.__connection)
+        return self.__appliance_device_read_community
+
+    @property
+    def appliance_device_snmp_v1_trap_destinations(self):
+        """
+        Gets the ApplianceDeviceSNMPv1TrapDestinations API client.
+
+        Returns:
+            ApplianceDeviceSNMPv1TrapDestinations:
+        """
+        if not self.__appliance_device_snmp_v1_trap_destinations:
+            self.__appliance_device_snmp_v1_trap_destinations = ApplianceDeviceSNMPv1TrapDestinations(self.__connection)
+        return self.__appliance_device_snmp_v1_trap_destinations
+
+    @property
+    def appliance_device_snmp_v3_trap_destinations(self):
+        """
+        Gets the ApplianceDeviceSNMPv3TrapDestinations API client.
+
+        Returns:
+            ApplianceDeviceSNMPv3TrapDestinations:
+        """
+        if not self.__appliance_device_snmp_v3_trap_destinations:
+            self.__appliance_device_snmp_v3_trap_destinations = ApplianceDeviceSNMPv3TrapDestinations(self.__connection)
+        return self.__appliance_device_snmp_v3_trap_destinations
+
+    @property
+    def appliance_device_snmp_v3_users(self):
+        """
+        Gets the ApplianceDeviceSNMPv3Users API client.
+
+        Returns:
+            ApplianceDeviceSNMPv3Users:
+        """
+        if not self.__appliance_device_snmp_v3_users:
+            self.__appliance_device_snmp_v3_users = ApplianceDeviceSNMPv3Users(self.__connection)
+        return self.__appliance_device_snmp_v3_users
+
     @property
     def appliance_node_information(self):
         """
@@ -1129,6 +1130,18 @@ class OneViewClient(object):
             self.__appliance_time_and_locale_configuration = ApplianceTimeAndLocaleConfiguration(self.__connection)
         return self.__appliance_time_and_locale_configuration
 
+    @property
+    def versions(self):
+        """
+        Gets the Version API client.
+
+        Returns:
+            Version:
+        """
+        if not self.__versions:
+            self.__versions = Versions(self.__connection)
+        return self.__versions
+
     @property
     def backups(self):
         """
@@ -1140,3 +1153,56 @@ class OneViewClient(object):
         if not self.__backups:
             self.__backups = Backups(self.__connection)
         return self.__backups
+
+    @property
+    def login_details(self):
+        """
+        Gets the login details
+
+        Returns:
+        List of login details
+        """
+        if not self.__login_details:
+            self.__login_details = LoginDetails(self.__connection)
+        return self.__login_details
+
+    @property
+    def licenses(self):
+        """
+        Gets all the licenses
+        Returns:
+        List of licenses
+        """
+        if not self.__licenses:
+            self.__licenses = Licenses(self.__connection)
+        return self.__licenses
+
+    @property
+    def hypervisor_managers(self):
+        """
+        Gets the Hypervisor Managers API client.
+
+        Returns:
+            HypervisorManagers
+        """
+        return HypervisorManagers(self.__connection)
+
+    @property
+    def certificates_server(self):
+        """
+        Gets the Certificates Server API client.
+
+        Returns:
+            Server Certificate:
+        """
+        return CertificatesServer(self.__connection)
+
+    @property
+    def hypervisor_cluster_profiles(self):
+        """
+        Gets the Hypervisor Cluster Profiles API client.
+
+        Returns:
+            Hypervisor Cluster Profiles:
+        """
+        return HypervisorClusterProfiles(self.__connection)
